/*
 * ====================================================================
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
package org.apache.http.client.cache;

import org.apache.http.HttpHost;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;

/**
 * Given a particular HttpRequest, flush any cache entries that this request
 * would invalidate.
 *
 * @since 4.3
 */
public interface HttpCacheInvalidator {

    /**
     * Remove cache entries from the cache that are no longer fresh or have been
     * invalidated in some way.
     *
     * @param host
     *            The backend host we are talking to
     * @param req
     *            The HttpRequest to that host
     */
    void flushInvalidatedCacheEntries(HttpHost host, HttpRequest req);

    /**
     * Flushes entries that were invalidated by the given response received for
     * the given host/request pair.
     */
    void flushInvalidatedCacheEntries(HttpHost host, HttpRequest request, HttpResponse response);

}
